<?php
/**
 * This file is part of NoiseLabs-SmartyBundle
 *
 * NoiseLabs-SmartyBundle is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * NoiseLabs-SmartyBundle is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with NoiseLabs-SmartyBundle; if not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2011-2016 Vítor Brandão
 *
 * @category    NoiseLabs
 * @package     SmartyBundle
 * @copyright   (C) 2011-2016 Vítor Brandão <vitor@noiselabs.org>
 * @license     http://www.gnu.org/licenses/lgpl-3.0-standalone.html LGPL-3
 * @link        http://www.noiselabs.org
 */

namespace NoiseLabs\Bundle\SmartyBundle\Tests\Extension;

use Assetic\Asset\AssetInterface;
use Assetic\AssetManager;
use Assetic\Factory\AssetFactory;
use Assetic\Filter\FilterInterface;
use Assetic\FilterManager;
use Assetic\ValueSupplierInterface;
use NoiseLabs\Bundle\SmartyBundle\Extension\AsseticExtension;
use NoiseLabs\Bundle\SmartyBundle\Tests\TestCase;

/**
 * Test suite for the assetic extension.
 *
 * Includes test based on:
 * - Assetic\Tests\Extension\Twig\AsseticExtensionTest [assetic/assetic]
 *
 * @author Vítor Brandão <vitor@noiselabs.org>
 */
class AsseticExtensionTest extends TestCase
{
    protected function setUp()
    {
        parent::setUp();

        if (!class_exists(AssetManager::class)) {
            $this->markTestSkipped('Assetic is not available.');
        }

        $templatesDir = realpath(__DIR__.'/../Assetic/templates');

        $this->am = $this->getMock(AssetManager::class);
        $this->fm = $this->getMock(FilterManager::class);

        $this->valueSupplier = $this->getMock(ValueSupplierInterface::class);

        $this->factory = new AssetFactory($templatesDir);
        $this->factory->setAssetManager($this->am);
        $this->factory->setFilterManager($this->fm);

        $this->engine->setTemplateDir($templatesDir);
        $this->engine->addExtension(new AsseticExtensionForTest($this->factory, false, $this->valueSupplier));
    }

    public function testExtensionName()
    {
        $extension = new AsseticExtensionForTest(new AssetFactory('/foo'));

        $this->assertEquals('assetic', $extension->getName());
    }

    public function testAbsolutePath()
    {
        $xml = $this->renderXml('absolute_path.smarty');
        $this->assertEquals(1, count($xml->asset));
        $this->assertStringStartsWith('css/', (string) $xml->asset['url']);
    }

    public function testFilters()
    {
        $filter = $this->getMock(FilterInterface::class);

        $this->fm->expects($this->at(0))
            ->method('get')
            ->with('foo')
            ->will($this->returnValue($filter));
        $this->fm->expects($this->at(1))
            ->method('get')
            ->with('bar')
            ->will($this->returnValue($filter));

        $xml = $this->renderXml('filters.smarty');
        $this->assertEquals(1, count($xml->asset));
        $this->assertStringStartsWith('css/', (string) $xml->asset['url']);
    }

    public function testOptionalFilter()
    {
        $filter = $this->getMock(FilterInterface::class);

        $this->fm->expects($this->once())
            ->method('get')
            ->with('foo')
            ->will($this->returnValue($filter));

        $xml = $this->renderXml('optional_filter.smarty');
        $this->assertEquals(1, count($xml->asset));
        $this->assertStringStartsWith('css/', (string) $xml->asset['url']);
    }

    public function testOutputPattern()
    {
        $xml = $this->renderXml('output_pattern.smarty');
        $this->assertEquals(1, count($xml->asset));
        $this->assertStringStartsWith('css/packed/', (string) $xml->asset['url']);
        $this->assertStringEndsWith('.css', (string) $xml->asset['url']);
    }

    public function testOutput()
    {
        $xml = $this->renderXml('output_url.smarty');
        $this->assertEquals(1, count($xml->asset));
        $this->assertEquals('explicit_url.css', (string) $xml->asset['url']);
    }

    public function testMixture()
    {
        $asset = $this->getMock(AssetInterface::class);
        $this->am->expects($this->any())
            ->method('get')
            ->with('foo')
            ->will($this->returnValue($asset));

        $xml = $this->renderXml('mixture.smarty');
        $this->assertEquals(1, count($xml->asset));
        $this->assertEquals('packed/mixture', (string) $xml->asset['url']);
    }

    public function testDebug()
    {
        $filter = $this->getMock(FilterInterface::class);

        $this->fm->expects($this->once())
            ->method('get')
            ->with('bar')
            ->will($this->returnValue($filter));

        $xml = $this->renderXml('debug.smarty');
        $this->assertEquals(2, count($xml->asset));
        $this->assertStringStartsWith('css/packed_', (string) $xml->asset[0]['url']);
        $this->assertStringEndsWith('.css', (string) $xml->asset[0]['url']);
    }

    public function testCombine()
    {
        $filter = $this->getMock(FilterInterface::class);

        $this->fm->expects($this->once())
            ->method('get')
            ->with('bar')
            ->will($this->returnValue($filter));

        $xml = $this->renderXml('combine.smarty');
        $this->assertEquals(1, count($xml->asset));
        $this->assertEquals('css/packed.css', (string) $xml->asset[0]['url']);
    }

    public function testImage()
    {
        $xml = $this->renderXml('image.smarty');
        $this->assertEquals(1, count($xml->image));
        $this->assertStringEndsWith('.png', (string) $xml->image[0]['url']);
    }

    public function testVariables()
    {
        $this->valueSupplier->expects($this->any())
            ->method('getValues')
            ->will($this->returnValue(array('foo' => 'a', 'bar' => 'b')));

        $xml = $this->renderXml('variables.smarty');
        $this->assertEquals(2, $xml->url->count());

        // formatting of the asset url changed with kriswallsmith/assetic 1.1.3
        $this->assertEquals('js/7d0828c.a.b_foo_1.js', (string) $xml->url[0]);
        $this->assertEquals('js/7d0828c.a.b_variable_input._2.js', (string) $xml->url[1]);
    }
}

class AsseticExtensionForTest extends AsseticExtension
{
    protected function getAssetUrl(AssetInterface $asset, array $options = array())
    {
        return $asset->getTargetPath();
    }
}
